@@ -5,10 +5,11 @@ from collections import defaultdict
 
 import requests
 import lxml.etree as ET
+import jinja2
 
 from freud_api_crawler.string_utils import clean_markup, extract_page_nr, always_https, normalize_white_space
 
-from freud_api_crawler.tei_utils import make_pb
+# from freud_api_crawler.tei_utils import make_pb
 
 FRD_BASE = "https://www.freud-edition.net"
 FRD_API = os.environ.get('FRD_API', f'{FRD_BASE}/jsonapi/')
@@ -366,8 +367,8 @@ class FrdManifestation(FrdClient):
             print(f"no content for manifestation_seite/{page_id}")
             print("#####################\n")
             body = "<p>BLANK</p>"
-        wrapped_body = f'<div xmlns="http://www.tei-c.org/ns/1.0" xml:id="page__{page_id}">{body}</div>'
-        cleaned_body = clean_markup(wrapped_body)
+        # wrapped_body = f'<div xmlns="http://www.tei-c.org/ns/1.0" xml:id="page__{page_id}">{body}</div>'
+        cleaned_body = clean_markup(body)
         cleaned_body = normalize_white_space(cleaned_body)
         faks = page_json['included'][0]
         page_nr = extract_page_nr(page_attributes['title'])
@@ -384,7 +385,7 @@ class FrdManifestation(FrdClient):
         }
         return result
 
-    def make_xml(self, save=False, limit=True):
+    def make_xml(self, save=False, limit=True, dump=False):
 
         """serializes a manifestation as XML/TEI document
 
@@ -393,153 +394,192 @@ class FrdManifestation(FrdClient):
 
         :return: A lxml.etree
         """
-        doc = self.tei_dummy
-        root_el = doc.xpath('//tei:TEI', namespaces=self.nsmap)[0]
-        root_el.attrib["{http://www.w3.org/XML/1998/namespace}base"] = "https://whatever.com"
-        root_el.attrib[
-            "{http://www.w3.org/XML/1998/namespace}id"
-        ] = f"manifestation__{self.manifestation_id}"
-        title = doc.xpath('//tei:title[@type="manifestation"]', namespaces=self.nsmap)[0]
-        title.text = f"{self.md__title} ({self.manifestation_signatur})"
-        if self.manifestation['data']['attributes']['field_shorttitle']:
-            s_title = doc.xpath('//tei:title[@type="manifestation_short"]', namespaces=self.nsmap)[0]
-            s_title.text = f"{self.manifestation['data']['attributes']['field_shorttitle']['value']}"
-        p_title = doc.xpath('//tei:title[@type="publication"]', namespaces=self.nsmap)[0]
-        p_rs = ET.Element("{http://www.tei-c.org/ns/1.0}rs")
-        p_rs.attrib["type"] = "bibl"
-        try:
-            p_rs.attrib["ref"] = f"#bibl__{self.publication['data']['id']}"
-        except TypeError or KeyError:
-            p_rs.attrib["ref"] = f"#bibl__{self.publication}"
-        try:
-            p_rs.text = f"{self.publication['data']['attributes']['title']}"
-        except TypeError or KeyError:
-            p_rs.text = f"{self.publication}"
-        p_title.append(p_rs)
-        w_title = doc.xpath('//tei:title[@type="work"]', namespaces=self.nsmap)[0]
-        w_rs = ET.Element("{http://www.tei-c.org/ns/1.0}rs")
-        w_rs.attrib["type"] = "bibl"
-        w_rs.attrib["ref"] = f"#bibl__{self.werk['id']}"
-        w_title.append(w_rs)
-        w_rs.text = f"{self.werk['attributes']['title']}"
-        titleStmt = doc.xpath('//tei:titleStmt', namespaces=self.nsmap)[0]
-        author = ET.Element("{http://www.tei-c.org/ns/1.0}author")
-        try:
-            author.text = f"{self.author['data']['attributes']['name']}"
-        except TypeError or KeyError:
-            author.text = f"{self.author}"
-        titleStmt.insert(3, author)
-        fileDesc = doc.xpath('//tei:fileDesc', namespaces=self.nsmap)[0]
-        sourceDesc = ET.Element("{http://www.tei-c.org/ns/1.0}sourceDesc")
-        bibl = ET.Element("{http://www.tei-c.org/ns/1.0}bibl")
-        if self.publication is not None:
+        if dump:
+            json_dump = {}
+            json_dump["id"] = f"manifestation__{self.manifestation_id}"
+            json_dump['man_title'] = f"{self.md__title} ({self.manifestation_signatur})"
             try:
-                bibl.attrib["{http://www.w3.org/XML/1998/namespace}id"] = f"bibl__{self.publication['data']['id']}"
-            except KeyError or TypeError:
-                return
+                s_title_t = self.manifestation['data']['attributes']['field_shorttitle']
+                json_dump['man_shorttitle'] = s_title_t['value']
+            except (KeyError, TypeError):
+                print("No short title found!")
+            json_dump["publication"] = {}
             try:
-                if self.publication['data']['attributes']['field_secondary_title']:
-                    bibl_title = ET.Element("{http://www.tei-c.org/ns/1.0}title")
-                    bibl_title.attrib['type'] = "sub"
-                    bibl_title.text = f"{self.publication['data']['attributes']['field_secondary_title']['value']}"
-                    bibl.append(bibl_title)
-            except KeyError or TypeError:
-                return
+                json_dump["publication"]["id"] = f"bibl__{self.publication['data']['id']}"
+            except (KeyError, TypeError):
+                json_dump["publication"]["id"] = f"bibl__{self.manifestation_id}"
             try:
-                if self.publication['data']['attributes']['field_shorttitle'] is not None:
-                    bibl_title = ET.Element("{http://www.tei-c.org/ns/1.0}title")
-                    bibl_title.attrib['type'] = "short"
-                    bibl_title.text = f"{self.publication['data']['attributes']['field_shorttitle']['value']}"
-                    bibl.append(bibl_title)
-            except KeyError or TypeError:
-                return
+                json_dump["publication"]["title"] = self.publication['data']['attributes']['title']
+            except (KeyError, TypeError):
+                json_dump["publication"]["title"] = self.manifestation_id
+            json_dump["work"] = {}
+            json_dump["work"]["id"] = f"bibl__{self.werk['id']}"
+            json_dump["work"]["title"] = self.werk['attributes']['title']
             try:
-                if self.publication['data']['attributes']['field_titel'] is not None:
-                    bibl_title = ET.Element("{http://www.tei-c.org/ns/1.0}title")
-                    bibl_title.attrib['type'] = "main"
-                    bibl_title.text = f"{self.publication['data']['attributes']['field_titel']['value']}"
-                    bibl.append(bibl_title)
-            except KeyError or TypeError:
-                return
+                json_dump["author"] = self.author['data']['attributes']['name']
+            except (KeyError, TypeError):
+                json_dump["author"] = "Freud, Sigmund"
+            try:
+                bibl_type = self.publication['data']['type'].replace('--', '/')
+                json_dump["publication"]["id"] = f"{bibl_type}/{self.publication['data']['id']}"
+            except (KeyError, TypeError):
+                print("No publication ID found!")
+            try:
+                bibl_title_obj = self.publication['data']['attributes']['field_titel']
+                json_dump["publication"]["title_main"] = bibl_title_obj['value']
+            except (KeyError, TypeError):
+                print("No publication main title found!")
+            try:
+                bibl_title_obj = self.publication['data']['attributes']['field_secondary_title']
+                json_dump["publication"]["title_sub"] = bibl_title_obj['value']
+            except (KeyError, TypeError):
+                print("No publication secodnary title found!")
+            try:
+                bibl_title_obj = self.publication['data']['attributes']['field_shorttitle']
+                json_dump["publication"]["title_short"] = bibl_title_obj['value']
+            except (KeyError, TypeError):
+                print("No publication short title found!")
             try:
                 places = self.publication['data']['attributes']['field_publication_place']
                 for x in places:
                     place = x["value"]
-                    bibl_place = ET.Element("{http://www.tei-c.org/ns/1.0}pubPlace")
-                    bibl_place.text = place
-                    bibl.append(bibl_place)
-            except KeyError or TypeError:
-                return
+                    json_dump["publication"]["places"] = []
+                    json_dump["publication"]["places"].append({"name": place})
+            except (KeyError, TypeError):
+                print("No publication place(s) found!")
             try:
-                if self.publication['data']['attributes']['field_publication_year'] is not None:
-                    bibl_date = ET.Element("{http://www.tei-c.org/ns/1.0}date")
-                    bibl_date.attrib['when'] = f"{self.publication['data']['attributes']['field_publication_year']}"
-                    bibl_date.text = f"{self.publication['data']['attributes']['field_publication_year']}"
-                    bibl.append(bibl_date)
-            except KeyError or TypeError:
-                return
+                bibl_date_obj = self.publication['data']['attributes']['field_publication_year']
+                json_dump["publication"]["date"] = bibl_date_obj
+            except (KeyError, TypeError):
+                print("No publication year found!")
             try:
-                if self.publication['data']['attributes']['field_band'] is not None:
-                    bibl_scope = ET.Element("{http://www.tei-c.org/ns/1.0}biblScope")
-                    bibl_scope.text = f"{self.publication['data']['attributes']['field_band']['value']}"
-                    bibl.append(bibl_scope)
-            except KeyError or TypeError:
-                return
-            if self.publisher is not None:
-                try:
-                    bibl_publisher = ET.Element("{http://www.tei-c.org/ns/1.0}publisher")
-                    bibl_publisher.text = self.publisher['data']['attributes']['name']
-                    bibl.append(bibl_publisher)
-                except KeyError or TypeError:
-                    return
-            if self.herausgeber is not None:
-                try:
-                    if type(self.herausgeber) is list:
-                        for x in self.herausgeber:
-                            bibl_publisher = ET.Element("{http://www.tei-c.org/ns/1.0}publisher")
-                            bibl_publisher.text = x['data']['attributes']['name']
-                            bibl.append(bibl_publisher)
-                    else:
-                        bibl_publisher = ET.Element("{http://www.tei-c.org/ns/1.0}author")
-                        bibl_publisher.text = self.herausgeber['data']['attributes']['name']
-                        bibl.append(bibl_publisher)
-                except KeyError or TypeError:
-                    return
-            if self.pub_author is not None:
-                try:
-                    if type(self.pub_author) is list:
-                        for x in self.pub_author:
-                            bibl_author = ET.Element("{http://www.tei-c.org/ns/1.0}author")
-                            bibl_author.text = x['data']['attributes']['name']
-                            bibl.append(bibl_author)
-                    else:
-                        bibl_author = ET.Element("{http://www.tei-c.org/ns/1.0}author")
-                        bibl_author.text = self.pub_author['data']['attributes']['name']
-                        bibl.append(bibl_author)
-                except KeyError or TypeError:
-                    return
-        sourceDesc.append(bibl)
-        fileDesc.insert(4, sourceDesc)
-        body = doc.xpath('//tei:body', namespaces=self.nsmap)[0]
-        pages = self.pages
-        if limit:
-            actual_pages = pages[:2]
-        else:
-            actual_pages = pages
-        for x in actual_pages:
-            page_json = self.get_page(x['id'])
-            pp = self.process_page(page_json)
-            div = ET.fromstring(pp['body'])
-            pb_el = make_pb(pp)
-            cur_div = div.xpath('//tei:div', namespaces=self.nsmap)[0]
-            cur_div.insert(0, pb_el)
-            body.append(div)
-        transform = ET.XSLT(self.xsl_doc)
-        tei = transform(doc)
-        if save:
+                bibl_scope_obj = self.publication['data']['attributes']['field_band']
+                json_dump["publication"]["biblScope"] = bibl_scope_obj['value']
+            except (KeyError, TypeError):
+                print("No publication field band found!")
+            try:
+                if type(self.publisher) is list:
+                    for x in self.publisher:
+                        pub_type = x['data']['type'].replace('--', '/')
+                        json_dump["publication"]["publisher"] = []
+                        json_dump["publication"]["publisher"].append(
+                            {
+                                "id": f"{pub_type}/{x['data']['id']}",
+                                "name": f"{x['data']['attributes']['name']} (field_publisher)"
+                            }
+                        )
+                else:
+                    pub_type = self.publisher['data']['type'].replace('--', '/')
+                    json_dump["publication"]["publisher"] = []
+                    json_dump["publication"]["publisher"] = [
+                        {
+                            "id": f"{pub_type}/{self.publisher['data']['id']}",
+                            "name": f"{self.publisher['data']['attributes']['name']} (field_publisher)"
+                        }
+                    ]
+            except (KeyError, TypeError):
+                print("No publication publisher found!")
+            try:
+                if type(self.herausgeber) is list:
+                    for x in self.herausgeber:
+                        pub_type = x['data']['type'].replace('--', '/')
+                        json_dump["publication"]["herausgeber"] = []
+                        json_dump["publication"]["herausgeber"].append(
+                            {
+                                "id": f"{pub_type}/{x['data']['id']}",
+                                "name": f"{x['data']['attributes']['name']} (field_herausgeber)"
+                            }
+                        )
+                else:
+                    pub_type = self.herausgeber['data']['type'].replace('--', '/')
+                    json_dump["publication"]["herausgeber"] = []
+                    json_dump["publication"]["herausgeber"].append(
+                        {
+                            "id": f"{pub_type}/{self.herausgeber['data']['id']}",
+                            "name": f"{self.herausgeber['data']['attributes']['name']} (field_herausgeber)"
+                        }
+                    )
+            except (KeyError, TypeError):
+                print("No publication herausgeber found!")
+            try:
+                if type(self.pub_author) is list:
+                    for x in self.pub_author:
+                        pub_type = x['data']['type'].replace('--', '/')
+                        json_dump["publication"]["author"] = []
+                        json_dump["publication"]["author"].append(
+                            {
+                                "id": f"{pub_type}/{x['data']['id']}",
+                                "name": f"{x['data']['attributes']['name']} (field_authors)"
+                            }
+                        )
+                else:
+                    pub_type = self.pub_author['data']['type'].replace('--', '/')
+                    json_dump["publication"]["author"] = []
+                    json_dump["publication"]["author"].append(
+                        {
+                            "id": f"{pub_type}/{self.pub_author['data']['id']}",
+                            "name": f"{self.pub_author['data']['attributes']['name']} (field_authors)"
+                        }
+                    )
+            except (KeyError, TypeError):
+                print("No publication author(s) found!")
+            try:
+                if type(self.pub_editors) is list:
+                    for x in self.pub_editors:
+                        pub_type = x['data']['type'].replace('--', '/')
+                        json_dump["publication"]["editor"] = []
+                        json_dump["publication"]["editor"].append(
+                            {
+                                "id": f"{pub_type}/{x['data']['id']}",
+                                "name": f"{x['data']['attributes']['name']} (field_editors)"
+                            }
+                        )
+                else:
+                    pub_type = self.pub_editors['data']['type'].replace('--', '/')
+                    json_dump["publication"]["editor"] = []
+                    json_dump["publication"]["editor"].append(
+                        {
+                            "id": f"{pub_type}/{self.pub_editors['data']['id']}",
+                            "name": f"{self.pub_editors['data']['attributes']['name']} (field_editors)"
+                        }
+                    )
+            except (KeyError, TypeError):
+                print("No publication editor(s) found!")
+            try:
+                msType = self.repository['data']['type']
+                json_dump["repository"] = {
+                    "id": f"{msType}__{self.repository['data']['id']}",
+                    "name": self.repository['data']['attributes']['name']
+                }
+            except (KeyError, TypeError):
+                print("It seems there is not 'filed_aufbewahrungsort'")
+            json_dump["pages"] = []
+            pages = self.pages
+            if limit:
+                actual_pages = pages[:2]
+            else:
+                actual_pages = pages
+            for x in actual_pages:
+                page_json = self.get_page(x['id'])
+                pp = self.process_page(page_json)
+                json_dump["pages"].append(pp)
             os.makedirs(os.path.join(self.save_dir, self.werk_signatur), exist_ok=True)
+            with open(self.save_path_json, 'w', encoding='utf8') as f:
+                json.dump(json_dump, f)
+        else:
+            with open(self.save_path_json, 'r', encoding='utf8') as f:
+                json_dump = json.load(f)
+        templateLoader = jinja2.FileSystemLoader(searchpath=["./freud_api_crawler", "."])
+        templateEnv = jinja2.Environment(loader=templateLoader)
+        template = templateEnv.get_template('./templates/tei.xml')
+        tei = template.render({"objects": [json_dump]})
+        tei = ET.fromstring(tei)
+        transform = ET.XSLT(self.xsl_doc)
+        tei = transform(tei)
+        # tei = ET.tostring(tei)
+        if save:
             with open(self.save_path, 'wb') as f:
-                f.write(ET.tostring(tei, pretty_print=True, encoding="utf-8"))
+                f.write(tei)
         return tei
 
     def get_fe_werk_signatur(self):
@@ -558,24 +598,26 @@ class FrdManifestation(FrdClient):
 
         :return: json
         """
-        if self.manifestation['data']['relationships'][field_type]['data'] is not None:
-            try:
-                item = self.manifestation['data']['relationships'][field_type]['data']
-                item_id = item['id']
-                node_type = item['type'].split('--')[1]
-                taxonomy = item['type'].split('--')[0]
-                url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
-                r = requests.get(
-                    url,
-                    cookies=self.cookie,
-                    allow_redirects=True
-                )
-                result = r.json()
-                return result
-            except KeyError or TypeError:
-                print(f"{field_type} is null")
-        else:
-            return
+        try:
+            item = self.manifestation['data']['relationships'][field_type]['data']
+        except (KeyError, TypeError):
+            print(f"looks like there is no {field_type}")
+            return {}
+        try:
+            item_id = item['id']
+        except (KeyError, TypeError):
+            print(f"looks like there is not ID for {field_type}")
+            return {}
+        node_type = item['type'].split('--')[1]
+        taxonomy = item['type'].split('--')[0]
+        url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
+        r = requests.get(
+            url,
+            cookies=self.cookie,
+            allow_redirects=True
+        )
+        result = r.json()
+        return result
 
     def get_fields_any_any(self, field_type, get_fields):
         """requests manifestation 'relationships' fields
@@ -584,40 +626,51 @@ class FrdManifestation(FrdClient):
 
         :return: json
         """
-        if get_fields is not None:
-            try:
-                field_any = get_fields
-                item = field_any['data']['relationships'][field_type]['data']
-                if type(item) is list:
-                    result = []
-                    for x in item:
-                        item_id = x['id']
-                        node_type = x['type'].split('--')[1]
-                        taxonomy = x['type'].split('--')[0]
-                        url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
-                        r = requests.get(
-                            url,
-                            cookies=self.cookie,
-                            allow_redirects=True
-                        )
-                        res = r.json()
-                        result.append(res)
-                else:
-                    item_id = item['id']
-                    node_type = item['type'].split('--')[1]
-                    taxonomy = item['type'].split('--')[0]
-                    url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
-                    r = requests.get(
-                        url,
-                        cookies=self.cookie,
-                        allow_redirects=True
-                    )
-                    result = r.json()
-                return result
-            except KeyError or TypeError:
-                print(f"{field_type} is null")
+        try:
+            get_fields['data']['id']
+        except (KeyError, TypeError):
+            print("looks like there is no ID for get_fields")
+            return {}
+        try:
+            item = get_fields['data']['relationships'][field_type]['data']
+        except (KeyError, TypeError):
+            print(f"looks like there is no {field_type}")
+            return {}
+        if type(item) is list:
+            result = []
+            for x in item:
+                try:
+                    item_id = x['id']
+                except (KeyError, TypeError):
+                    print(f"looks like there is not ID for {field_type}")
+                    return {}
+                node_type = x['type'].split('--')[1]
+                taxonomy = x['type'].split('--')[0]
+                url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
+                r = requests.get(
+                    url,
+                    cookies=self.cookie,
+                    allow_redirects=True
+                )
+                res = r.json()
+                result.append(res)
+            return result
         else:
-            return
+            try:
+                item_id = item['id']
+            except (KeyError, TypeError):
+                print(f"looks like there is not ID for {field_type}")
+                return {}
+            node_type = item['type'].split('--')[1]
+            taxonomy = item['type'].split('--')[0]
+            url = f"{self.endpoint}{taxonomy}/{node_type}/{item_id}"
+            r = requests.get(
+                url,
+                cookies=self.cookie,
+                allow_redirects=True
+            )
+            result = r.json()
+            return result
 
     def __init__(
         self,
@@ -629,26 +682,13 @@ class FrdManifestation(FrdClient):
         self.manifestation_endpoint = f"{self.endpoint}node/manifestation/{manifestation_id}"
         self.manifestation = self.get_manifest()
         self.werk = self.manifestation['included'][0]
-        try:
-            self.publication = self.get_fields_any('field_published_in')
-        except TypeError or KeyError:
-            self.publication = self.manifestation_id
-        try:
-            self.author = self.get_fields_any('field_authors')
-        except TypeError or KeyError:
-            self.author = "Freud, Sigmund"
-        try:
-            self.publisher = self.get_fields_any_any('field_publisher', self.publication)
-        except TypeError or KeyError:
-            self.publisher = None
-        try:
-            self.herausgeber = self.get_fields_any_any('field_herausgeber', self.publication)
-        except TypeError or KeyError:
-            self.herausgeber = None
-        try:
-            self.pub_author = self.get_fields_any_any('field_authors', self.publication)
-        except TypeError or KeyError:
-            self.pub_author = None
+        self.publication = self.get_fields_any('field_published_in')
+        self.author = self.get_fields_any('field_authors')
+        self.repository = self.get_fields_any('field_aufbewahrungsort')
+        self.publisher = self.get_fields_any_any('field_publisher', self.publication)
+        self.herausgeber = self.get_fields_any_any('field_herausgeber', self.publication)
+        self.pub_author = self.get_fields_any_any('field_authors', self.publication)
+        self.pub_editors = self.get_fields_any_any('field_editors', self.publication)
         self.werk_folder = self.werk['attributes']['path']['alias']
         # self.manifestation_folder = self.manifestation['attributes']['path']['alias']
         self.man_attrib = self.manifestation['data']['attributes']
@@ -668,9 +708,13 @@ class FrdManifestation(FrdClient):
         self.werk_signatur = self.get_fe_werk_signatur()
         self.manifestation_signatur = f"{self.werk_signatur}{self.man_attrib['field_signatur_sfe_type']}"
         self.file_name = f"sfe-{self.manifestation_signatur.replace('/', '__').replace('.', '_')}.xml"
+        self.file_name_json = f"sfe-{self.manifestation_signatur.replace('/', '__').replace('.', '_')}.json"
         self.save_path = os.path.join(
             self.save_dir, self.werk_signatur, self.file_name
         )
+        self.save_path_json = os.path.join(
+            self.save_dir, self.werk_signatur, self.file_name_json
+        )
 
 
 def yield_works(url, simple=True):
